// Get search term, if one exists
var utilities = {}, u = utilities;
utilities.searchTerm = function () {
    var match = window.location.href.match(/(search_query|q)=([^&]+)/);
    if (!match) return false;
    return decodeURIComponent(match[2]);
}

// Official "location" of the user (stats tab or search page)
utilities.location = function () {
    var adapter = {
        graph: 'historical',
        twitterFeed: 'tweets',
        trends: 'trending'
    }, adapted = adapter[localStorage.view];

    return this.searchTerm() ? 'search' : (adapted || localStorage.view);
}

// Get whether or not use just upgraded, then remove value
utilities.pruneUpgraded = function () {
    var value = localStorage.justUpgraded;
    if (value) delete localStorage.justUpgraded;
    return value;
}

utilities.safely = function (callback) {
    try { return callback(); } catch (e) { log('safely caught error', e); }
    return null;
}

function forceNumber(input) {
    if (typeof(input) === 'number') return input;
    if (typeof(input) === 'string') input = input.replace(/[^0-9\.]/g, '');
    if (!input) input = 0;
    return parseFloat(input, 10);
}

function formatNumber(input, options) {
    if(input == 'N/A' || (typeof input === 'string' && input.indexOf('<') > -1) )
        return input;
    input = forceNumber(input);
    var finalNumber;
    var suffix = '';
    if(input > 1000000000) {
        finalNumber = Number(input/1000000000).toFixed(1);
        suffix = 'B';
    }else if(input > 1000000) {
        finalNumber = Number(input/1000000).toFixed(1);
        suffix = 'M';
    }else if(input >= 10000 || ( options && options.size === 'smallest' && input >= 1000 ) ) {
        finalNumber = Number(input/1000).toFixed(1);
        suffix = 'K';
        // remove small decimals
        finalStr = String(finalNumber);
        var dot = finalStr.indexOf('.');
        if(dot > -1) {
            var decimal = finalStr.substr(dot + 1);
            if(decimal[0] === '1' || decimal[0] === '9') {
                finalNumber = Math.round(finalNumber);
            }
        }
    }else if(input >= 1000) {
        finalNumber = Math.round(input);
    }else if(String(input).indexOf('.') > -1) {
        var parts = String(input).split('.');
        if(parts[1][1] > 5)
            parts[1][0]++;
        finalNumber = parts[0] + '.' + parts[1][0];
    }else{
        finalNumber = input;
    }
    // format decimal nicely
    var finalStr = String(finalNumber);
    if(finalStr.match(/^[0-9]*\.[0-9]*0$/)) {
        finalStr = finalStr.substr(0, finalStr.length - 1);
        if(finalStr[finalStr.length - 1] === '.')
            finalStr = finalStr.substr(0, finalStr.length - 1);
    }
    // add commas where needed
    if(suffix === '')
        finalStr = commaNumber(finalNumber);
    return finalStr + suffix;
}

function commaNumber(input) {
    if(typeof input === 'string')
        return input;
    input = forceNumber(input);
    return Number(input).toLocaleString('en');
}
var numberWithCommas = commaNumber;

utilities.commaNumber = commaNumber;

var idd = 0;
Function.prototype.multiCb = function() {
    var count = 0;
    var already_ran = false;
    var callback = this;
    var data = {};
    var runCheck;
    var id = idd++;
    var object = function(name) {
        clearTimeout(runCheck);
        count++;
        return function() {
            count--;
            if(name) {
                data[name] = arguments;
            }
            if(count === 0) {
                clearTimeout(runCheck);
                runCheck = setTimeout(function() {
                    if(already_ran)
                        return;
                    else
                        already_ran = true;
                    callback(data);                    
                }, 100);
            }
        };
    };

    object.timeout = function(ms) {
        setTimeout(function() {
            if(already_ran)
                return;
            else
                already_ran = true;
            callback(data);
        }, ms);
    };

    return object;
    
};

function calculatePercentage(num1, num2) {
    if (!num1) num1 = 0;
    if (!num2) num2 = 0;

    num1 = num1.toString().split(",").join("");
    num2 = num2.toString().split(",").join("");

    num1 = Math.abs(parseFloat(num1, 10));
    num2 = Math.abs(parseFloat(num2, 10));

    var p = ((num1 / num2) * 100).toFixed(1);
    if (isNaN(p) || !isFinite(p)) p = null;
    if(!isNaN(p) && p !== null && typeof p !== 'undefined')
        return p;
    else
        return 0;
}

function colorInterpolate(score) {
    if (!score || isNaN(score)) return [0, 0, 0];

    var gradient = [
        [200, 50, 0],  // Red
        [230, 190, 0], // Yellow
        [100, 200, 0]  // Green
    ];
    var max = 100;
    var interval = max/gradient.length; // 33.333
    var point = score/interval; // 60/33.333 = 1.8

    var index = Math.floor(point); // floor(1.8) => 1
    var ratio = point-index; // 1.8 - 1 = 0.8
    if (index > 0) index --; // 0

    var a = gradient[index]; // gradient[0] => Red
    var b = gradient[Math.min(index+1, gradient.length-1)]; // gradient[1] => Yellow
    var c = [];

    for (var i=0;i<a.length;i++) {
        c[i] = Math.round(a[i] + (b[i] - a[i])*ratio);
    }

    return c;
}

function runAtTimes(fn /*, ... */) {
    for(var i = 1; i < arguments.length; i++)
        setTimeout(fn, arguments[i]);
}

function pageScope(fn, argument) {
    var argument = JSON.stringify(argument);
    pageExec('(' + fn.toString() + ')(' + argument + ')');
}

function pageExec(code) {
    var script2 = document.createElement("script");
    script2.textContent = "localStorage.inject_value = " + code + ";";
    document.documentElement.appendChild(script2);
    var ret = localStorage.inject_value;
    delete localStorage.inject_value;
    return ret;
}

function copyText(txt, callback) {
    if (window.isFirefox) {
        return _cb({ msg: 'copy', text: txt }, function(data) {
            if (callback) callback();
        }); 
    }
    chrome.extension.sendMessage({ copy: txt });
    callback();
}

function copyTagsGeneral(tags, appendTo) {
    copyText(tags, function () {
        log('Send copy message to background page');
        var copied = $('<div>', { class: "copied" }).text("Tags Copied!").hide();
        if (appendTo) appendTo.append(copied.fadeIn('slow'));
        setTimeout(function() {
            copied.fadeOut('slow');
            setTimeout(function(){
                copied.remove();
            }, 1000);
        }, 3000);
    });
}

var htmlEncoder = (function() {
    var charToEntityRegex,
        entityToCharRegex,
        charToEntity,
        entityToChar;

    function resetCharacterEntities() {
        charToEntity = {};
        entityToChar = {};
        // add the default set
        addCharacterEntities({
            '&amp;'     :   '&',
            '&gt;'      :   '>',
            '&lt;'      :   '<',
            '&quot;'    :   '"',
            '&#39;'     :   "'"
        });
    }

    function addCharacterEntities(newEntities) {
        var charKeys = [],
            entityKeys = [],
            key, echar;
        for (key in newEntities) {
            echar = newEntities[key];
            entityToChar[key] = echar;
            charToEntity[echar] = key;
            charKeys.push(echar);
            entityKeys.push(key);
        }
        charToEntityRegex = new RegExp('(' + charKeys.join('|') + ')', 'g');
        entityToCharRegex = new RegExp('(' + entityKeys.join('|') + '|&#[0-9]{1,5};' + ')', 'g');
    }

    function htmlEncode(value){
        var htmlEncodeReplaceFn = function(match, capture) {
            return charToEntity[capture];
        };

        return (!value) ? value : String(value).replace(charToEntityRegex, htmlEncodeReplaceFn);
    }

    function htmlDecode(value) {
        var htmlDecodeReplaceFn = function(match, capture) {
            return (capture in entityToChar) ? entityToChar[capture] : String.fromCharCode(parseInt(capture.substr(2), 10));
        };

        return (!value) ? value : String(value).replace(entityToCharRegex, htmlDecodeReplaceFn);
    }

    resetCharacterEntities();

    return {
        htmlEncode: htmlEncode,
        htmlDecode: htmlDecode
    };
})();
